@@ -5,11 +5,12 @@ Custom element classes related to text, such as paragraph (CT_P) and runs
 (CT_R).
 """
 
-from ..enum.text import WD_UNDERLINE
+from ..enum.text import WD_ALIGN_PARAGRAPH, WD_UNDERLINE
 from .ns import qn
 from .simpletypes import ST_BrClear, ST_BrType
 from .xmlchemy import (
-    BaseOxmlElement, OptionalAttribute, ZeroOrMore, ZeroOrOne
+    BaseOxmlElement, OptionalAttribute, OxmlElement, RequiredAttribute,
+    ZeroOrMore, ZeroOrOne
 )
 
 
@@ -21,6 +22,13 @@ class CT_Br(BaseOxmlElement):
     clear = OptionalAttribute('w:clear', ST_BrClear)
 
 
+class CT_Jc(BaseOxmlElement):
+    """
+    ``<w:jc>`` element, specifying paragraph justification.
+    """
+    val = RequiredAttribute('w:val', WD_ALIGN_PARAGRAPH)
+
+
 class CT_P(BaseOxmlElement):
     """
     ``<w:p>`` element, containing the properties and text for a paragraph.
@@ -32,6 +40,39 @@ class CT_P(BaseOxmlElement):
         self.insert(0, pPr)
         return pPr
 
+    def add_p_before(self):
+        """
+        Return a new ``<w:p>`` element inserted directly prior to this one.
+        """
+        new_p = OxmlElement('w:p')
+        self.addprevious(new_p)
+        return new_p
+
+    @property
+    def alignment(self):
+        """
+        The value of the ``<w:jc>`` grandchild element or |None| if not
+        present.
+        """
+        pPr = self.pPr
+        if pPr is None:
+            return None
+        return pPr.alignment
+
+    @alignment.setter
+    def alignment(self, value):
+        pPr = self.get_or_add_pPr()
+        pPr.alignment = value
+
+    def clear_content(self):
+        """
+        Remove all child elements, except the ``<w:pPr>`` element if present.
+        """
+        for child in self[:]:
+            if child.tag == qn('w:pPr'):
+                continue
+            self.remove(child)
+
     def set_sectPr(self, sectPr):
         """
         Unconditionally replace or add *sectPr* as a grandchild in the
@@ -79,12 +120,31 @@ class CT_PPr(BaseOxmlElement):
     )
     pStyle = ZeroOrOne('w:pStyle')
     numPr = ZeroOrOne('w:numPr', successors=__child_sequence__[7:])
+    jc = ZeroOrOne('w:jc', successors=__child_sequence__[27:])
     sectPr = ZeroOrOne('w:sectPr', successors=('w:pPrChange',))
 
     def _insert_pStyle(self, pStyle):
         self.insert(0, pStyle)
         return pStyle
 
+    @property
+    def alignment(self):
+        """
+        The value of the ``<w:jc>`` child element or |None| if not present.
+        """
+        jc = self.jc
+        if jc is None:
+            return None
+        return jc.val
+
+    @alignment.setter
+    def alignment(self, value):
+        if value is None:
+            self._remove_jc()
+            return
+        jc = self.get_or_add_jc()
+        jc.val = value
+
     @property
     def style(self):
         """
@@ -117,6 +177,8 @@ class CT_R(BaseOxmlElement):
     rPr = ZeroOrOne('w:rPr')
     t = ZeroOrMore('w:t')
     br = ZeroOrMore('w:br')
+    cr = ZeroOrMore('w:cr')
+    tab = ZeroOrMore('w:tab')
     drawing = ZeroOrMore('w:drawing')
 
     def _insert_rPr(self, rPr):
@@ -141,6 +203,14 @@ class CT_R(BaseOxmlElement):
         drawing.append(inline_or_anchor)
         return drawing
 
+    def clear_content(self):
+        """
+        Remove all child elements except the ``<w:rPr>`` element if present.
+        """
+        content_child_elms = self[1:] if self.rPr is not None else self[:]
+        for child in content_child_elms:
+            self.remove(child)
+
     @property
     def style(self):
         """
@@ -161,6 +231,28 @@ class CT_R(BaseOxmlElement):
         rPr = self.get_or_add_rPr()
         rPr.style = style
 
+    @property
+    def text(self):
+        """
+        A string representing the textual content of this run, with content
+        child elements like ``<w:tab/>`` translated to their Python
+        equivalent.
+        """
+        text = ''
+        for child in self:
+            if child.tag == qn('w:t'):
+                text += child.text
+            elif child.tag == qn('w:tab'):
+                text += '\t'
+            elif child.tag in (qn('w:br'), qn('w:cr')):
+                text += '\n'
+        return text
+
+    @text.setter
+    def text(self, text):
+        self.clear_content()
+        _RunContentAppender.append_to_run_from_text(self, text)
+
     @property
     def underline(self):
         """
@@ -284,3 +376,59 @@ class CT_Underline(BaseOxmlElement):
 
         val = WD_UNDERLINE.to_xml(value)
         self.set(qn('w:val'), val)
+
+
+class _RunContentAppender(object):
+    """
+    Service object that knows how to translate a Python string into run
+    content elements appended to a specified ``<w:r>`` element. Contiguous
+    sequences of regular characters are appended in a single ``<w:t>``
+    element. Each tab character ('\t') causes a ``<w:tab/>`` element to be
+    appended. Likewise a newline or carriage return character ('\n', '\r')
+    causes a ``<w:cr>`` element to be appended.
+    """
+    def __init__(self, r):
+        self._r = r
+        self._bfr = []
+
+    @classmethod
+    def append_to_run_from_text(cls, r, text):
+        """
+        Create a "one-shot" ``_RunContentAppender`` instance and use it to
+        append the run content elements corresponding to *text* to the
+        ``<w:r>`` element *r*.
+        """
+        appender = cls(r)
+        appender.add_text(text)
+
+    def add_text(self, text):
+        """
+        Append the run content elements corresponding to *text* to the
+        ``<w:r>`` element of this instance.
+        """
+        for char in text:
+            self.add_char(char)
+        self.flush()
+
+    def add_char(self, char):
+        """
+        Process the next character of input through the translation finite
+        state maching (FSM). There are two possible states, buffer pending
+        and not pending, but those are hidden behind the ``.flush()`` method
+        which must be called at the end of text to ensure any pending
+        ``<w:t>`` element is written.
+        """
+        if char == '\t':
+            self.flush()
+            self._r.add_tab()
+        elif char in '\r\n':
+            self.flush()
+            self._r.add_cr()
+        else:
+            self._bfr.append(char)
+
+    def flush(self):
+        text = ''.join(self._bfr)
+        if text:
+            self._r.add_t(text)
+        del self._bfr[:]
