@@ -8,21 +8,23 @@ from __future__ import (
     absolute_import, division, print_function, unicode_literals
 )
 
-from docx.enum.text import WD_BREAK, WD_UNDERLINE
+from docx.enum.text import WD_ALIGN_PARAGRAPH, WD_BREAK, WD_UNDERLINE
+from docx.oxml import OxmlElement
+from docx.oxml.ns import qn
 from docx.oxml.text import CT_P, CT_R
 from docx.text import Paragraph, Run
 
 import pytest
 
-from mock import call, Mock
-
+from .oxml.parts.unitdata.document import a_body
 from .oxml.unitdata.text import (
-    a_b, a_bCs, a_br, a_caps, a_cs, a_dstrike, a_p, a_shadow, a_smallCaps,
-    a_snapToGrid, a_specVanish, a_strike, a_t, a_u, a_vanish, a_webHidden,
-    an_emboss, an_i, an_iCs, an_imprint, an_oMath, a_noProof, an_outline,
-    an_r, an_rPr, an_rStyle, an_rtl
+    a_b, a_bCs, a_br, a_caps, a_cr, a_cs, a_dstrike, a_jc, a_p, a_pPr,
+    a_pStyle, a_shadow, a_smallCaps, a_snapToGrid, a_specVanish, a_strike,
+    a_t, a_tab, a_u, a_vanish, a_webHidden, an_emboss, an_i, an_iCs,
+    an_imprint, an_oMath, a_noProof, an_outline, an_r, an_rPr, an_rStyle,
+    an_rtl
 )
-from .unitutil import class_mock, instance_mock
+from .unitutil import call, class_mock, instance_mock, Mock
 
 
 class DescribeParagraph(object):
@@ -40,6 +42,15 @@ class DescribeParagraph(object):
         assert isinstance(run, Run)
         assert run._r is paragraph._p.r_lst[0]
 
+    def it_knows_its_alignment_value(self, alignment_get_fixture):
+        paragraph, expected_value = alignment_get_fixture
+        assert paragraph.alignment == expected_value
+
+    def it_can_change_its_alignment_value(self, alignment_set_fixture):
+        paragraph, value, expected_xml = alignment_set_fixture
+        paragraph.alignment = value
+        assert paragraph._p.xml == expected_xml
+
     def it_knows_its_paragraph_style(self):
         cases = (
             (Mock(name='p_elm', style='foobar'), 'foobar'),
@@ -60,9 +71,29 @@ class DescribeParagraph(object):
             p.style = style
             assert p_elm.style == expected_setting
 
-    def it_knows_the_text_it_contains(self, text_prop_fixture):
-        p, expected_text = text_prop_fixture
-        assert p.text == expected_text
+    def it_knows_the_text_it_contains(self, text_get_fixture):
+        paragraph, expected_text = text_get_fixture
+        assert paragraph.text == expected_text
+
+    def it_can_replace_the_text_it_contains(self, text_set_fixture):
+        paragraph, text, expected_text = text_set_fixture
+        paragraph.text = text
+        assert paragraph.text == expected_text
+
+    def it_can_insert_a_paragraph_before_itself(self, insert_before_fixture):
+        paragraph, text, style, body, expected_xml = insert_before_fixture
+        new_paragraph = paragraph.insert_paragraph_before(text, style)
+        assert isinstance(new_paragraph, Paragraph)
+        assert new_paragraph.text == text
+        assert new_paragraph.style == style
+        assert body.xml == expected_xml
+
+    def it_can_remove_its_content_while_preserving_formatting(
+            self, clear_fixture):
+        paragraph, expected_xml = clear_fixture
+        _paragraph = paragraph.clear()
+        assert paragraph._p.xml == expected_xml
+        assert _paragraph is paragraph
 
     # fixtures -------------------------------------------------------
 
@@ -81,6 +112,107 @@ class DescribeParagraph(object):
         expected_xml = a_p().with_nsdecls().with_child(r_bldr).xml()
         return paragraph, text, style, expected_xml
 
+    @pytest.fixture(params=[
+        ('center', WD_ALIGN_PARAGRAPH.CENTER),
+        (None,     None),
+    ])
+    def alignment_get_fixture(self, request):
+        jc_val, expected_alignment_value = request.param
+        p_bldr = a_p().with_nsdecls()
+        if jc_val is not None:
+            p_bldr.with_child(a_pPr().with_child(a_jc().with_val(jc_val)))
+        p = p_bldr.element
+        paragraph = Paragraph(p)
+        return paragraph, expected_alignment_value
+
+    @pytest.fixture(params=[
+        ('left', WD_ALIGN_PARAGRAPH.CENTER, 'center'),
+        ('left', None,                      None),
+        (None,   WD_ALIGN_PARAGRAPH.LEFT,   'left'),
+        (None,   None,                      None),
+    ])
+    def alignment_set_fixture(self, request):
+        initial_jc_val, new_alignment_value, expected_jc_val = request.param
+        # paragraph --------------------
+        p_bldr = a_p().with_nsdecls()
+        if initial_jc_val is not None:
+            p_bldr.with_child(
+                a_pPr().with_child(
+                    a_jc().with_val(initial_jc_val))
+            )
+        p = p_bldr.element
+        paragraph = Paragraph(p)
+        # expected_xml -----------------
+        pPr_bldr = a_pPr()
+        if expected_jc_val is not None:
+            pPr_bldr.with_child(a_jc().with_val(expected_jc_val))
+        p_bldr = a_p().with_nsdecls().with_child(pPr_bldr)
+        expected_xml = p_bldr.xml()
+        return paragraph, new_alignment_value, expected_xml
+
+    @pytest.fixture
+    def clear_fixture(self, request):
+        """
+        After XML should be before XML with content removed. So snapshot XML
+        after adding formatting but before adding content to get after XML.
+        """
+        style, text = ('Heading1', 'foo\tbar')
+        p = OxmlElement('w:p')
+        # expected_xml -----------------
+        if style is not None:
+            p.style = style
+        expected_xml = p.xml
+        # paragraph --------------------
+        paragraph = Paragraph(p)
+        if text is not None:
+            paragraph.add_run(text)
+        return paragraph, expected_xml
+
+    @pytest.fixture
+    def insert_before_fixture(self):
+        text, style = 'foobar', 'Heading1'
+        body = (
+            a_body().with_nsdecls().with_child(
+                a_p())
+        ).element
+        p = body.find(qn('w:p'))
+        paragraph = Paragraph(p)
+        expected_xml = (
+            a_body().with_nsdecls().with_child(
+                a_p().with_child(
+                    a_pPr().with_child(
+                        a_pStyle().with_val(style))).with_child(
+                    an_r().with_child(
+                        a_t().with_text(text)))).with_child(
+                a_p())
+        ).xml()
+        return paragraph, text, style, body, expected_xml
+
+    @pytest.fixture
+    def runs_fixture(self, p_, Run_, r_, r_2_, runs_):
+        paragraph = Paragraph(p_)
+        run_, run_2_ = runs_
+        return paragraph, Run_, r_, r_2_, run_, run_2_
+
+    @pytest.fixture
+    def text_get_fixture(self):
+        p = (
+            a_p().with_nsdecls().with_child(
+                an_r().with_child(
+                    a_t().with_text('foo'))).with_child(
+                an_r().with_child(
+                    a_t().with_text(' de bar')))
+        ).element
+        paragraph = Paragraph(p)
+        return paragraph, 'foo de bar'
+
+    @pytest.fixture
+    def text_set_fixture(self):
+        p = a_p().with_nsdecls().element
+        paragraph = Paragraph(p)
+        paragraph.add_run('barfoo')
+        return paragraph, 'foo\tbar\rbaz\n', 'foo\tbar\nbaz\n'
+
     # fixture components ---------------------------------------------
 
     @pytest.fixture
@@ -113,24 +245,6 @@ class DescribeParagraph(object):
         run_2_ = instance_mock(request, Run, name='run_2_')
         return run_, run_2_
 
-    @pytest.fixture
-    def runs_fixture(self, p_, Run_, r_, r_2_, runs_):
-        paragraph = Paragraph(p_)
-        run_, run_2_ = runs_
-        return paragraph, Run_, r_, r_2_, run_, run_2_
-
-    @pytest.fixture
-    def text_prop_fixture(self):
-        p = (
-            a_p().with_nsdecls().with_child(
-                an_r().with_child(
-                    a_t().with_text('foo'))).with_child(
-                an_r().with_child(
-                    a_t().with_text(' de bar')))
-        ).element
-        paragraph = Paragraph(p)
-        return paragraph, 'foo de bar'
-
 
 class DescribeRun(object):
 
@@ -178,10 +292,27 @@ class DescribeRun(object):
         run.add_break(break_type)
         assert run._r.xml == expected_xml
 
-    def it_knows_the_text_it_contains(self, text_prop_fixture):
-        run, expected_text = text_prop_fixture
+    def it_can_add_a_tab(self, add_tab_fixture):
+        run, expected_xml = add_tab_fixture
+        run.add_tab()
+        assert run._r.xml == expected_xml
+
+    def it_can_remove_its_content_while_preserving_formatting(
+            self, clear_fixture):
+        run, expected_xml = clear_fixture
+        _run = run.clear()
+        assert run._r.xml == expected_xml
+        assert _run is run
+
+    def it_knows_the_text_it_contains(self, text_get_fixture):
+        run, expected_text = text_get_fixture
         assert run.text == expected_text
 
+    def it_can_replace_the_text_it_contains(self, text_set_fixture):
+        run, text, expected_xml = text_set_fixture
+        run.text = text
+        assert run._r.xml == expected_xml
+
     # fixtures -------------------------------------------------------
 
     @pytest.fixture(params=[
@@ -205,6 +336,11 @@ class DescribeRun(object):
         expected_xml = an_r().with_nsdecls().with_child(br_bldr).xml()
         return run, break_type, expected_xml
 
+    @pytest.fixture
+    def add_tab_fixture(self, run):
+        expected_xml = an_r().with_nsdecls().with_child(a_tab()).xml()
+        return run, expected_xml
+
     @pytest.fixture(params=['foobar', ' foo bar', 'bar foo '])
     def add_text_fixture(self, request, run, Text_):
         text_str = request.param
@@ -338,6 +474,22 @@ class DescribeRun(object):
         expected_xml = an_r().with_nsdecls().with_child(rPr_bldr).xml()
         return run, bool_prop_name, value, expected_xml
 
+    @pytest.fixture(params=[
+        ('bi', 'foobar'), ('bi', None), ('', 'foobar'), ('', None)
+    ])
+    def clear_fixture(self, request):
+        formatting, text = request.param
+        r = OxmlElement('w:r')
+        if 'b' in formatting:
+            r.get_or_add_rPr()._add_b().val = True
+        if 'i' in formatting:
+            r.get_or_add_rPr()._add_i().val = True
+        expected_xml = r.xml
+        if text is not None:
+            r.add_t(text)
+        run = Run(r)
+        return run, expected_xml
+
     @pytest.fixture(params=['Foobar', None])
     def style_get_fixture(self, request):
         style = request.param
@@ -359,15 +511,32 @@ class DescribeRun(object):
         expected_xml = self.r_bldr_with_style(after_style).xml()
         return run, after_style, expected_xml
 
-    @pytest.fixture
-    def text_prop_fixture(self, Text_):
-        r = (
-            an_r().with_nsdecls().with_child(
-                a_t().with_text('foo')).with_child(
-                a_t().with_text('bar'))
-        ).element
-        run = Run(r)
-        return run, 'foobar'
+    @pytest.fixture(params=[
+        (('',),                          ''),
+        (('xfoobar',),                   'foobar'),
+        (('bpage', 'xabc', 'xdef', 't'), '\nabcdef\t'),
+        (('xabc', 't', 'xdef', 'n'),     'abc\tdef\n'),
+    ])
+    def text_get_fixture(self, request):
+        content_children, expected_text = request.param
+        r_bldr = self.r_content_bldr(content_children)
+        run = Run(r_bldr.element)
+        return run, expected_text
+
+    @pytest.fixture(params=[
+        ('abc',      ('xabc',)),
+        ('abc\tdef', ('xabc', 't', 'xdef')),
+        ('abc\rdef', ('xabc', 'n', 'xdef')),
+    ])
+    def text_set_fixture(self, request):
+        text, expected_elm_codes = request.param
+        # starting run contains text, so can tell if it doesn't get replaced
+        r_bldr = self.r_content_bldr(('xfoobar'))
+        run = Run(r_bldr.element)
+        # expected_xml -----------------
+        r_bldr = self.r_content_bldr(expected_elm_codes)
+        expected_xml = r_bldr.xml()
+        return run, text, expected_xml
 
     @pytest.fixture(params=[
         (None,     None),
@@ -409,11 +578,6 @@ class DescribeRun(object):
 
     # fixture components ---------------------------------------------
 
-    @pytest.fixture
-    def run(self):
-        r = an_r().with_nsdecls().element
-        return Run(r)
-
     def r_bldr_with_style(self, style):
         rPr_bldr = an_rPr()
         if style is not None:
@@ -428,6 +592,28 @@ class DescribeRun(object):
         r_bldr = an_r().with_nsdecls().with_child(rPr_bldr)
         return r_bldr
 
+    def r_content_bldr(self, elm_codes):
+        """
+        Return a ``<w:r>`` builder having child elements indicated by
+        *elm_codes*.
+        """
+        r_bldr = an_r().with_nsdecls()
+        for e in elm_codes:
+            if e.startswith('x'):
+                r_bldr.with_child(a_t().with_text(e[1:]))
+            elif e == 't':
+                r_bldr.with_child(a_tab())
+            elif e.startswith('b'):
+                r_bldr.with_child(a_br().with_type(e[1:]))
+            elif e == 'n':
+                r_bldr.with_child(a_cr())
+        return r_bldr
+
+    @pytest.fixture
+    def run(self):
+        r = an_r().with_nsdecls().element
+        return Run(r)
+
     @pytest.fixture
     def Text_(self, request):
         return class_mock(request, 'docx.text.Text')
